<?php
/**
 * Class to build HTML forms
 * @author José Carlos Cruz Parra AKA internia
 * josecarlos@programadorphpfreelance.com
 * http://programadorphpfreelance.com
 * 2008-09-06
 * 2011-06-11
 * 2011-12-06
 * 2012-04-07
 * This code is released under the GNU General Public License.
 */
class FormBuilder
{
	private $_id = "";
	private $_name = "";
	private $_action = "";
	private $_method = "get";
	private $_items = array();
	private $_textAccept = "Aceptar";
	private $_textCancel = "Cancelar";
	private $_textYes = "Sí";
	private $_textNo = "No";
	private $_cancelurl = "";

	private $_divstyle = "text-align:center";
	private $_tablestyle = "margin:0px auto;text-align:left";

	private $_classclaro = "listadoclaro";
	private $_classnormal = "listadonormal";

	public function __construct($id="", $name="")
	{
		$this->_id = empty($id) ? FormBuilder::GenRandomStr() : $id;
		$this->_name = empty($name) ? $this->_id : $name;
	}

	private static function GenRandomStr()
	{
		$minlen = 6;
		$maxlen = 9;
		$validchars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

		$len = mt_rand($minlen, $maxlen);
		$max = strlen($validchars)-1;
		$str = "";
		for($c=0; $c<$len; $c++)
		{
			$str .= substr($validchars, mt_rand(0, $max), 1);
		}
		return $str;
	}

	public function SetAction($str)
	{
		$this->_action = $str;
	}

	public function SetMethod($str)
	{
		$this->_method = $str;
	}

	public function SetItems($arr)
	{
		$this->_items = $arr;
	}

	public function SetTextAccept($str)
	{
		$this->_textAccept = $str;
	}

	public function SetTextCancel($str)
	{
		$this->_textCancel = $str;
	}

	public function SetTextYes($str)
	{
		$this->_textYes = $str;
	}

	public function SetTextNo($str)
	{
		$this->_textNo = $str;
	}

	public function SetCancelUrl($str)
	{
		$this->_cancelurl = $str;
	}

	public function SetClassClaro($str)
	{
		$this->_classclaro = $str;
	}

	public function SetClassNormal($str)
	{
		$this->_classnormal = $str;
	}

	public function GetHTML($items=null)
	{
		if(!empty($items) && is_array($items))
		{
			$this->_items = $items;
		}

		$html = "";
		$hayfile = false;
		$hiddens = "";
		foreach($this->_items as $grupo)
		{
			$html .= sprintf("<tr>\n	<th colspan=\"2\">%s</th>\n</tr>\n", $grupo["concepto"]);
			$c = 0;
			foreach($grupo["data"] as $data)
			{
				$exploded = explode(";", html_entity_decode($data["info"]));
				$info = array();
				foreach($exploded as $aux)
				{
					$aux = explode("=", str_replace('=>', '[*>*]', $aux), 2);
					$info[$aux[0]] = str_replace('[*>*]', '=>', $aux[1]);
				}

				$permiso_id = isset($data["permiso_id"]) ? $data["permiso_id"] : SWPHP_PERMISO_LECTURAYESCRITURA;
				switch($permiso_id)
				{
					//Acceso prohibido
					case 0:
						//Nada
						break;
					//Sólo lectura
					case 1:
						$html .= sprintf("<tr class=\"%s\">\n\t<td>%s%s</td>\n\t<td>", $c % 2 ? $this->_classnormal : $this->_classclaro, $data["texto"], (isset($info["type"]) && $info["type"] == "file" && $info["img"] && is_file($data["valor"])) ? " <a class=\"external\" title=\"Ver a tamaño real\" href=\"{$data["valor"]}\"><img src=\"{$data["valor"]}\" alt=\"\" width=\"100\" /></a>" : "");
						$html .= $this->Info2ReadOnly($info, $data["valor"], $hayfile);
						$html .= "</td>\n</tr>\n";
						$c++;
						break;
					//Lectura y escritura
					case 2:
						//sigue
					//Acceso total
					case 3:
						if($info["tag"] == "input" && $info["type"] == "hidden")
						{
							$hiddens .= "<input type=\"hidden\" class=\"{$info["class"]}\" name=\"{$grupo["concepto"]}[{$data["variable"]}]\" value=\"{$data["valor"]}\" />\n";
						}
						else
						{
							$html .= sprintf("<tr class=\"%s\">\n\t<td>%s%s</td>\n\t<td>", $c % 2 ? $this->_classnormal : $this->_classclaro, $data["texto"], (isset($info["type"]) && $info["type"] == "file" && $info["img"] && is_file($data["valor"])) ? " <a class=\"external\" title=\"Ver a tamaño real\" href=\"{$data["valor"]}\"><img src=\"{$data["valor"]}\" alt=\"\" width=\"100\" /></a>" : "");
							$html .= FormBuilder::Info2Html($info, "{$grupo["concepto"]}[{$data["variable"]}]", $data["valor"], $hayfile);
							$html .= "</td>\n</tr>\n";
							$c++;
						}
						break;
				}
			}
		}

		return sprintf("<form id=\"%s\" name=\"%s\" action=\"%s\" method=\"%s\" %s>\n<div style=\"%s\">\n<table style=\"%s\">\n",
				$this->_id,
				$this->_name,
				$this->_action,
				$this->_method,
				$hayfile ? "enctype=\"multipart/form-data\"" : "",
				$this->_divstyle,
				$this->_tablestyle
			)
			.$html
			.sprintf("<tr>\n\t<td colspan=\"2\" style=\"text-align:center\"><br />\n"
				."\t\t<input type=\"submit\" value=\"%s\" class=\"boton\" />"
				."&nbsp;<input type=\"button\" value=\"%s\" class=\"boton\" onclick=\"%s\" />\n"
				."%s\t</td>\n</tr>\n</table>\n</div>\n</form>\n",
				$this->_textAccept,
				$this->_textCancel,
				empty($this->_cancelurl) ? "history.back()" : "document.location='{$this->_cancelurl}'",
				$hiddens
			);
	}

	public function Info2ReadOnly($info, $valor, &$hayfile)
	{
		$html = '';

		if(is_string($info))
		{
			$info = FormBuilder::Info2array($info);
		}

		switch($info["tag"])
		{
			case "input":
				switch($info["type"])
				{
					case "file":
					case "text":
						$html .= $valor;
						break;
					case "checkbox":
						$html .= $valor ? SWPHP::_Icon('check.png', 16, array('alt'=>$this->_textYes)).' '.$this->_textYes : SWPHP::_Icon('remove.png', 16, array('alt'=>$this->_textNo)).' '.$this->_textNo;
						break;
				}
				break;
			case "select":
				if(isset($info["options"]))
				{
					$pos = strpos($info["options"], '(');
					if($pos)
					{
						$func = substr($info["options"], 0, $pos);
						if($func == 'array' || function_exists($func) || strpos($func, '::') || strpos($func, '->'))
						{
							$options = eval("return {$info["options"]};");
						}
					}

					if(isset($options) && is_array($options))
					{
						foreach($options as $key=>$value)
						{
							if($key == $valor)
							{
								$html .= $value;
								break;
							}
						}
					}
				}
				else
				{
					$html .= $valor;
				}
				break;
			case "textarea":
				$html .= $valor;
				break;
		}

		return $html;
	}

	public static function Info2Html($info, $name, $valor, &$hayfile)
	{
		$html = '';

		if(is_string($info))
		{
			$info = FormBuilder::Info2array($info);
		}

		switch($info["tag"])
		{
			case "input":
				if($info["type"] == "radio")
				{
					if(isset($info["options"]))
					{
						$pos = strpos($info["options"], '(');
						if($pos)
						{
							$func = substr($info["options"], 0, $pos);
							if($func == 'array' || function_exists($func) || strpos($func, '::') || strpos($func, '->'))
							{
								$options = eval("return {$info["options"]};");
							}
						}

						if(isset($options) && is_array($options))
						{
							foreach($options as $key=>$value)
							{
								$html .= sprintf("<input type=\"radio\" name=\"%s\" id=\"%s_%s\" value=\"%s\" %s /><label for=\"%s_%s\"> %s</label>\n", $name, $name, $key, $key, $key == $valor ? 'checked="checked"' : '', $name, $key, $value);
							}
						}
					}
					else
					{
						$html .= sprintf("<input type=\"radio\" name=\"%s\" id=\"%s\" value=\"%s\" /><label for=\"%s\"> %s</label>\n", $name, $name, $valor, $name, $valor);
					}
				}
				else
				{
					$html .= "<input type=\"{$info["type"]}\"";
					if(isset($info['class']) && $info['class'])
					{
						$html .= " class=\"{$info["class"]}\"";
					}
					$html .= " name=\"{$name}\" ";
					if(!isset($info['id']) || $info['id'] == '')
					{
						$html .= " id=\"{$name}\"";
					}
					switch($info["type"])
					{
						case "file":
							$hayfile = true;
						case "text":
							$html .= " size=\"{$info["size"]}\" value=\"{$valor}\"";
							if(isset($info["maxlength"]))
							{
								$html .= " maxlength=\"{$info["maxlength"]}\"";
							}
							break;
						case "checkbox":
							if($valor)
							{
								$html .= " checked=\"checked\"";
							}
							break;
					}
					$html .= FormBuilder::Info2extras($info)."/>\n";
				}
				break;
			case "select":
				$html .= "<select name=\"{$name}\" ".FormBuilder::Info2extras($info);
				if(!isset($info['id']) || $info['id'] == '')
				{
					$html .= " id=\"{$name}\"";
				}
				$html .= ">\n";
				if(isset($info["options"]))
				{
					$pos = strpos($info["options"], '(');
					if($pos)
					{
						$func = substr($info["options"], 0, $pos);
						if($func == 'array' || function_exists($func) || strpos($func, '::') || strpos($func, '->'))
						{
							$options = eval("return {$info['options']};");
						}
					}

					if(isset($options) && is_array($options))
					{
						foreach($options as $key=>$value)
						{
							$html .= sprintf("<option value=\"%s\" %s >%s</option>\n", $key, $key == $valor ? 'selected="selected"' : '', $value);
						}
					}
				}
				else
				{
					$html .= "<option value=\"{$valor}\">{$valor}</option>\n";
				}
				$html .= "</select>\n";
				break;
			case "textarea":
				$html .= "<textarea";
				if(isset($info['class']) && $info['class'])
				{
					$html .= " class=\"{$info["class"]}\"";
				}
				if(!isset($info['id']) || $info['id'] == '')
				{
					$html .= " id=\"{$name}\"";
				}
				$html .= " name=\"{$name}\" cols=\"{$info["cols"]}\" rows=\"{$info["rows"]}\" ".FormBuilder::Info2extras($info).">{$valor}</textarea>\n";
				break;
		}

		return $html;
	}

	private static function Info2extras($info)
	{
		$extras = array(
			'id',
			'onkeydown',
			'onkeyup',
			'onkeypress',
			'onchange',
			'onblur',
			'title',
			'readonly',
			'disabled',
			);
		$html = '';

		if(is_string($info))
		{
			$info = FormBuilder::Info2array($info);
		}

		foreach($info as $key=>$value)
		{
			$key = strtolower($key);
			if(in_array($key, $extras))
			{
				$html .= sprintf('%s="%s" ', $key, $value);
			}
		}

		return $html;
	}

	private static function Info2array($info)
	{
		$items = explode(";", $info);
		$info = array();
		foreach($items as $item)
		{
			$item = explode("=", str_replace('=>', '[*>*]', $item), 2);
			$info[$item[0]] = str_replace('[*>*]', '=>', $item[1]);
		}

		return $info;
	}
}
?>