<?php
/**
 * Clase para poner palabras en plural
 * @author José Carlos Cruz Parra AKA internia
 * josecarlos@programadorphpfreelance.com
 * http://programadorphpfreelance.com
 * 2008-2015
 * This code is released under the GNU General Public License.
 */
class Pluralizador
{
    static function Plural($que, $considerarPalabras=true, $idioma='ES')
    {
        switch(mb_strtoupper($idioma))
        {
            case 'ES':
                return Pluralizador::Plural_ES($que, $considerarPalabras);
            default:
                return Pluralizador::Plural_EN($que, $considerarPalabras);
        }
    }

    static function Plural_ES($que, $considerarPalabras=true)
    {
        if($considerarPalabras)
        {
            $nomas = false;
            $return = '';
            $palabras = explode(' ', $que);
            for($c = 0, $size = sizeof($palabras); $c < $size; $c++)
            {
                if($c > 0 && $palabras[$c] == 'de')
                {
                    $nomas = true;
                }
                $palabra = $nomas ? $palabras[$c] : Pluralizador::Plural($palabras[$c], false);
                $return .= ($c < $size - 1) ? $palabra.' ' : $palabra;
            }

            return $return;
        }
        elseif(mb_substr(mb_strtolower($que), -2) == 'er')
		{
			return self::Plural_EN($que, $considerarPalabras);
		}
		else
        {
			$que = trim($que);

			$lastChar = mb_substr($que, -1);
			switch($lastChar)
            {
                case 'a':
                case 'e':
                case 'i':
                case 'o':
                case 'u':
				case 'á':
                case 'é':
                case 'í':
                case 'ó':
                case 'ú':
					return $que.'s';
                case 'A':
                case 'E':
                case 'I':
                case 'O':
                case 'U':
                case 'Á':
                case 'É':
                case 'Í':
                case 'Ó':
                case 'Ú':
                    return $que.'S';

                case 's':
                case 'S':
                    switch(mb_substr($que, -2, 1))
                    {
                        case 'á':
							$que = mb_substr($que, 0, -2).'a'.mb_substr($que, -1);
                            break;
                        case 'é':
                            $que = mb_substr($que, 0, -2).'e'.mb_substr($que, -1);
                            break;
                        case 'í':
                            $que = mb_substr($que, 0, -2).'i'.mb_substr($que, -1);
                            break;
                        case 'ó':
                            $que = mb_substr($que, 0, -2).'o'.mb_substr($que, -1);
                            break;
                        case 'ú':
                            $que = mb_substr($que, 0, -2).'u'.mb_substr($que, -1);
                            break;
                        case 'Á':
                            $que = mb_substr($que, 0, -2).'A'.mb_substr($que, -1);
                            break;
                        case 'É':
                            $que = mb_substr($que, 0, -2).'E'.mb_substr($que, -1);
                            break;
                        case 'Í':
                            $que = mb_substr($que, 0, -2).'I'.mb_substr($que, -1);
                            break;
                        case 'Ó':
                            $que = mb_substr($que, 0, -2).'O'.mb_substr($que, -1);
                            break;
                        case 'Ú':
                            $que = mb_substr($que, 0, -2).'U'.mb_substr($que, -1);
                            break;
                        default:
                            return $que;
                    }
                    return (mb_strtoupper($que) == $que) ? "{$que}ES" : "{$que}es";

				default:
					if($lastChar && mb_stripos('lrndzjsx', $lastChar) === false)
					{
						return (mb_strtoupper($que) == $que) ? "{$que}S" : "{$que}s";
					}
					else
					{
						switch($lastChar)
						{
							case 'z':
								if(mb_stripos('aeiouáéíóú', mb_substr($que, -2, 1)) !== false)
								{
									$que = mb_substr($que, 0, -1).'c';
								}
								break;
							case 'Z':
								if(mb_stripos('aeiouáéíóú', mb_substr($que, -2, 1)) !== false)
								{
									$que = mb_substr($que, 0, -1).'C';
								}
								break;
						}

						switch(mb_substr($que, -2, 1))
						{
							case 'á':
								$que = mb_substr($que, 0, -2).'a'.mb_substr($que, -1);
								break;
							case 'é':
								$que = mb_substr($que, 0, -2).'e'.mb_substr($que, -1);
								break;
							case 'í':
								$que = mb_substr($que, 0, -2).'i'.mb_substr($que, -1);
								break;
							case 'ó':
								$que = mb_substr($que, 0, -2).'o'.mb_substr($que, -1);
								break;
							case 'ú':
								$que = mb_substr($que, 0, -2).'u'.mb_substr($que, -1);
								break;
							case 'Á':
								$que = mb_substr($que, 0, -2).'A'.mb_substr($que, -1);
								break;
							case 'É':
								$que = mb_substr($que, 0, -2).'E'.mb_substr($que, -1);
								break;
							case 'Í':
								$que = mb_substr($que, 0, -2).'I'.mb_substr($que, -1);
								break;
							case 'Ó':
								$que = mb_substr($que, 0, -2).'O'.mb_substr($que, -1);
								break;
							case 'Ú':
								$que = mb_substr($que, 0, -2).'U'.mb_substr($que, -1);
								break;
						}
						return (mb_strtoupper($que) == $que) ? "{$que}ES" : "{$que}es";
					}
            }
        }
    }

    static function Plural_EN($que, $considerarPalabras = true)
    {
        if($considerarPalabras)
        {
            $return = '';
            $palabras = explode(' ', $que);
            for($c = 0, $size = sizeof($palabras); $c < $size; $c++)
            {
                $palabra = ($c < $size - 1) ? $palabras[$c] : Pluralizador::Plural($palabras[$c], false, 'EN');
                $return .= ($c < $size - 1) ? $palabra.' ' : $palabra;
            }

            return $return;
        }
        else
        {
            //TODO: Esto es básico, falta mucho por hacer

			$que = trim($que);

			switch($que[mb_strlen($que) - 1])
            {
				case 'y':
                case 'Y':
                    return (mb_strtoupper($que) == $que) ? mb_substr($que, 0, strlen($que)) . 'IES' : mb_substr($que, 0, strlen($que)) . 'ies';
                case 'a':
                case 'e':
                case 'i':
                case 'o':
                case 'u':
                case 'A':
                case 'E':
                case 'I':
                case 'O':
                case 'U':
                default:
                    return (mb_strtoupper($que) == $que) ? "{$que}S" : "{$que}s";
            }
        }
    }
	
	static function Singular($que, $considerarPalabras=true, $idioma='ES')
	{
		switch(strtoupper($idioma))
		{
			case 'ES':
				return Pluralizador::Singular_ES($que, $considerarPalabras);
			default:
				return Pluralizador::Singular_EN($que, $considerarPalabras);
		}
	}

	static function Singular_ES($que, $considerarPalabras=true)
	{
		if($considerarPalabras)
		{
			$nomas = false;
			$return = '';
			$palabras = explode(' ', $que);
			for($c=0,$size=sizeof($palabras); $c<$size; $c++)
			{
				if($c > 0 && $palabras[$c] == 'de')
				{
					$nomas = true;
				}

				$palabra = $nomas ? $palabras[$c] : Pluralizador::Singular_ES($palabras[$c], false);
				$return .= ($c < $size-1) ? "$palabra " : "$palabra";
			}
			return $return;
		}
		else
		{
			//TODO: Esto es básico, falta mucho por hacer
			switch($que[strlen($que)-1])
			{
				case 's':
				case 'S':
					return substr($que, 0, -1);
				default:
					return $que;
			}
		}
	}

	static function Singular_EN($que, $considerarPalabras=true)
	{
		if($considerarPalabras)
		{
			$return = "";
			$palabras = explode(" ", $que);
			for($c=0,$size=sizeof($palabras); $c<$size; $c++)
			{
				$palabra = ($c < $size-1) ? $palabras[$c] : Pluralizador::Singular_EN($palabras[$c], false);
				$return .= ($c < $size-1) ? "$palabra " : "$palabra";
			}
			return $return;
		}
		else
		{
			//TODO: Esto es básico, falta mucho por hacer
			switch($que[strlen($que)-1])
			{
				case 's':
				case 'S':
					return substr($que, 0, -1);
				default:
					return $que;
			}
		}
	}
}
