<?php
/**
 * Class to display paginated results from sql queries with extra functions
 * @author José Carlos Cruz Parra AKA internia
 * josecarlos@programadorphpfreelance.com
 * http://programadorphpfreelance.com
 * Basado en DB_Paginador
 * 2012-04-19
 * This code is released under the GNU General Public License.
 */
class SWPHP_Paginador
{
	/**
	 * Valores
	 *
	 * @var array
	 */
	private $_values = array(
		'key'=>'',
		'rpp'=>25,
		'desde'=>0,
		'numpags'=>0,
		'total'=>0,
		'regs'=>array(),
		'num_regs'=>0,
		'totales'=>array(),
		);
	/**
	 * Parámetros
	 *
	 * @var array
	 */
	private $_params = array(
		'method'=>'get',
		'completarConVacios'=>false,
		'usarImg'=>true,
		'img_dir'=>'',
		'img_orderASC_off'=>'',
		'img_orderASC_on'=>'',
		'img_orderDESC_off'=>'',
		'img_orderDESC_on'=>'',
		'img_anterior'=>'',
		'img_siguiente'=>'',
		'img_primera'=>'',
		'img_ultima'=>'',
		'img_buscar'=>'',
		'txt_orderASC'=>'',
		'txt_orderDESC'=>'',
		'titulo'=>'',
		'formato_tabla'=>array(),
		'etiqueta'=>'',
		'mostraretiqueta'=>true,
		'onclick'=>'',
		'tr_title'=>'',
		'orderBy'=>'',
		'permitirOrdenacion'=>true,
		'applyToRegs'=>'',
		'buscador'=>false,
		'filtro_campo'=>'',
		'filtro_valor'=>'',
		'filtro_exacto'=>false,
		'input_valor_size'=>12,
		'mostrarTotales'=>false,
		'action'=>'index.php',
		'tr_class_especiales'=>array(),
		'previoHTML'=>'',
		'extraHTML'=>'',
		'campos'=>array(),
		'camposEstrictos'=>false,
		'pag'=>1,
		);
	/**
	 * Conexión a la base de datos
	 *
	 * @var DB_Connection
	 */
	private $_db = null;
	/**
	 * Constructor
	 *
	 * @param string $key
	 * @param int $rpp
	 * @param DB_Connection $db
	 * @return SWPHP_Paginador
	 */
	public function __construct($key=null, $rpp=25, $db_=null)
	{
		if(is_null($key))
		{
			$key = sprintf('%06s', mt_rand(0, 100000));
		}

		if(is_null($db_))
		{
			global $db;

			$db_ = $db ? $db : DB_Connection::Factory();
		}

		$this->_values['key'] = $key;
		$this->_values['rpp'] = $rpp;
		$this->_db = $db_;
		$this->Reset();
	}
	/**
	 * Establece el valor de un parámetro
	 *
	 * @param string $key
	 * @param mixed $value
	 * @return bool
	 */
	public function SetParam($key, $value)
	{
		if(isset($this->_params[$key]))
		{
			$this->_params[$key] = $value;

			return true;
		}

		return false;
	}
	/**
	 * Establece el valor de un grupo de parámetros
	 *
	 * @param array $params
	 * @return bool
	 */
	public function SetParams($params)
	{
		$ok = true;

		foreach($params as $key=>$value)
		{
			$ok &= $this->SetParam($key, $value);
		}

		return $ok;
	}
	/**
	 * Obtiene el valor de un parámetro
	 *
	 * @param string $key
	 * @return mixed
	 */
	public function GetParam($key)
	{
		if(isset($this->_params[$key]))
		{
			return $this->_params[$key];
		}

		return false;
	}
	/**
	 * Obtiene el valor de un valor
	 *
	 * @param string $key
	 * @return mixed
	 */
	public function GetValue($key)
	{
		if(isset($this->_values[$key]))
		{
			return $this->_values[$key];
		}

		return false;
	}
	/**
	 * Establece los valores por defecto de los parámetros
	 *
	 * @return unknown
	 */
	public function Reset()
	{
		global $swphp_administracion;

		return $this->SetParams(array(
			'method'=>'get',
			'completarConVacios'=>false,
			'usarImg'=>true,
			'img_dir'=>SWPHP_THEME_DIR.'img/',
			'img_orderASC_off'=>'orderASC_off.gif',
			'img_orderASC_on'=>'orderASC_on.gif',
			'img_orderDESC_off'=>'orderDESC_off.gif',
			'img_orderDESC_on'=>'orderDESC_on.gif',
			'img_anterior'=>'pag_anterior.gif',
			'img_siguiente'=>'pag_siguiente.gif',
			'img_primera'=>'pag_primera.gif',
			'img_ultima'=>'pag_ultima.gif',
			'img_buscar'=>'icons/16x16/search.png',
			'txt_orderASC'=>'&uarr;',
			'txt_orderDESC'=>'&darr;',
			'titulo'=>'',
			'formato_tabla'=>array('style'=>'margin:0px auto;text-align:left','class'=>'paginador','id'=>'paginador_'.$this->_values['key']),
			'etiqueta'=>l('resultado'),
			'mostraretiqueta'=>true,
			'onclick'=>'',
			'tr_title'=>l('Seleccionar...'),
			'orderBy'=>'',
			'permitirOrdenacion'=>true,
			'applyToRegs'=>'',
			'buscador'=>false,
			'filtro_campo'=>'',
			'filtro_valor'=>'',
			'filtro_exacto'=>false,
			'input_valor_size'=>12,
			'mostrarTotales'=>false,
			'action'=>$swphp_administracion ? SWPHP_ADMIN_DIR.'index.php' : 'index.php',
			'tr_class_especiales'=>array(),
			'previoHTML'=>'',
			'extraHTML'=>'',
			'campos'=>array(),
			'pag'=>1,
			));
	}
	/**
	 * Llena los resultados
	 *
	 * @param mixed $from
	 */
	public function Fill($from)
	{
		if(is_array($from))
		{
			$this->FillFromArray($from);
		}
		else
		{
			$this->FillFromQuery($from);
		}
	}
	/**
	 * Llena los resultados a partir de una consulta SQL
	 *
	 * @param string $query
	 */
	public function FillFromQuery($query)
	{
		$request = $this->_GetRequest();

		$ssq = new SqlSelectQuery($query);
		$contenidoSelect = $ssq->GetContenidoComponente('SELECT');
		$modifiersSelect = $ssq->GetModifiersComponente('SELECT');
		$ssq->SetComponente('SELECT', '1', ''); //Para la consulta del total
		$ssq->SetContenidoComponente('LIMIT', '');
		//Filtro
		$this->_params['filtro_campo'] = (isset($request['filtro_campo']) && $request['filtro_campo']) ? $request['filtro_campo'] : false;
		$this->_params['filtro_valor'] = (isset($request['filtro_valor'])) ? $request['filtro_valor'] : false;
		$this->_params['filtro_exacto'] = (isset($request['filtro_exacto']) && $request['filtro_exacto']);
		if($this->_params['filtro_campo'] !== false && (strlen($this->_params['filtro_valor']) > 0 || $this->_params['filtro_exacto']))
		{
			$where = $ssq->GetContenidoComponente('WHERE');
			$filtro = $this->_params['filtro_exacto'] ? "= _utf8 '".addcslashes(utf8_encode($this->_params['filtro_valor']), "'")."' COLLATE utf8_general_ci " : "LIKE _utf8 '%".addcslashes(utf8_encode($this->_params['filtro_valor']), "'")."%' COLLATE utf8_general_ci ";
			$ssq->SetContenidoComponente('WHERE', ($where == '' ? '' : "{$where} AND")." (".$this->_params['filtro_campo']." {$filtro})");
		}

		$this->_values['total'] = $this->_db->GetCount($ssq->BuildQuery());
		if($this->_values['rpp'] == 0)
		{
			$this->_values['rpp'] = $this->_values['total'];
		}
		if($this->_values['total'] > 0)
		{
			//Ordenación
			$this->_values['orderBy'] = (isset($request['por']) && $request['por']) ? $request['por']." ".$request['modo'] : "";
			$order = $ssq->GetContenidoComponente('ORDER BY');
			if($this->_values['orderBy'] != "")
			{
				$ssq->SetContenidoComponente('ORDER BY', $order == '' ? $this->_values['orderBy'] : $this->_values['orderBy'].','.$order);
			}

			//Desde
			$this->_Desde(isset($request['pag']) ? intval($request['pag']) : 1);
			$ssq->SetContenidoComponente('LIMIT', "{$this->_values['desde']}, {$this->_values['rpp']}");

			//Obtener regs
			$ssq->SetComponente('SELECT', $contenidoSelect, $modifiersSelect); //Restauramos el select
			$this->_values['regs'] = $this->_db->GetRows($ssq->BuildQuery());
		}
		$this->_values['num_regs'] = sizeof($this->_values['regs']);

		if($this->_params['mostrarTotales'])
		{
			$ssq->SetContenidoComponente('LIMIT', '');
			$ssq->SetContenidoComponente('ORDER BY', '');
			$this->_values['totales'] = array();
			for($c=0,$size=sizeof($this->_params['campos']); $c<$size; $c++)
			{
				if(!$this->_params['campos'][$c]['hidden'])
				{
					if($this->_params['campos'][$c]['total'])
					{
						$ssq->SetContenidoComponente('SELECT', $this->_params['campos'][$c]['total']);
						$this->_values['totales'][$c] = $this->_db->GetValueQuery($ssq->BuildQuery());
					}
					else
					{
						$this->_values['totales'][$c] = false;
					}
				}
			}
		}
	}
	/**
	 * Llena el paginador a partir de un array de arrays asociativos (a modo de registros).
	 *
	 * @param array $array
	 */
	public function FillFromArray($array)
	{
		$this->_values['total'] = sizeof($array);
		if($this->_values['total'] > 0)
		{
			//Ordenación
			$request = $this->_GetRequest();
			if(isset($request['por']) && $request['por'])
			{
				$por = array();
				foreach($array as $key => $row)
				{
					if(isset($row[$request['por']]))
					{
						$por[$key]  = $row[$request['por']];
					}
				}
				array_multisort($por, $request['modo'] == "DESC" ? SORT_DESC : SORT_ASC, $array);
			}

			//Desde
			$this->_Desde(isset($request['pag']) ? intval($request['pag']) : 1);

			//Obtener regs
			$keys = array_keys($array);
			$size = $this->_values['rpp'] > 0 ? $this->_values['desde'] + $this->_values['rpp'] : $this->_values['total'] - $this->_values['desde'];
			for($c=$this->_values['desde']; $c<$size; $c++)
			{
				if(isset($keys[$c]) && isset($array[$keys[$c]]))
				{
					$this->_values['regs'][] = $array[$keys[$c]];
				}
			}
		}
		$this->_values['num_regs'] = sizeof($this->_values['regs']);
	}
	/**
	 * Añade un registro a los resultados
	 *
	 * @param array $array
	 */
	public function AddReg($array)
	{
		$this->_values['regs'][] = $array;
		$this->_values['num_regs']++;
	}
	/**
	 * Obtiene el conjunto de parámetros enviados, según el método del formulario
	 *
	 * @return array
	 */
	private function _GetRequest()
	{
		$excluir = array('sub', 'ssub');
		$request = $this->_params['method'] == "post" ? $_POST : $_GET;

		if(isset($_SESSION['SWPHP_Paginador'][$this->_values['key']]['request']))
		{
			foreach($_SESSION['SWPHP_Paginador'][$this->_values['key']]['request'] as $key=>$value)
			{
				if(in_array($key, $excluir))
				{
					unset($_SESSION['SWPHP_Paginador'][$this->_values['key']]['request'][$key]);
				}
				elseif(!isset($request[$key]))
				{
					$request[$key] = $value;
				}
			}
		}

		foreach($request as $key=>$value)
		{
			if(!in_array($key, $excluir))
			{
				$_SESSION['SWPHP_Paginador'][$this->_values['key']]['request'][$key] = $request[$key];
			}
		}

		return $request;
	}
	/**
	 * Si se ha asignado, aplica la función de usuario con el nombre $this->applyToReg
	 * a cada uno de los registros del listado
	 */
	public function Regs_Walk()
	{
		if($this->_params['applyToRegs'] != "")
		{
			array_walk($this->_values['regs'], $this->_params['applyToRegs'], $this); //Antes de 2009-01-17 el tercer parámetro era $this->_values['num_regs']);
		}
	}
	/**
	 * Sólo se usa desde el tpl,
	 * para completar con registros vacíos hasta el número de registros por página
	 * (si está activado y es necesario)
	 *
	 */
	public function CompletarConVacios()
	{
		if($this->_params['completarConVacios'])
		{
			//Completamos con registros vacíos hasta el número máximo de registros por página
			for($c=sizeof($this->_values['regs']),$size=$this->_values['rpp']; $c<$size; $c++)
			{
				$this->_values['regs'][$c] = array();
				$keys = array_keys($this->_values['regs'][0]);
				for($k=0, $size2=sizeof($keys); $k < $size2; $k++)
				{
					$this->_values['regs'][$c][$keys[$k]] = "&nbsp;";
				}
			}
		}
	}
	/**
	 * Para determinar el valor de desde
	 *
	 */
	private function _Desde($pag=1)
	{
		$numpags = ceil($this->_values['total'] / $this->_values['rpp']);

		if($pag > $numpags)
		{
			$pag = $numpags;
		}
		if($pag < 1)
		{
			$pag = 1;
		}

		$this->_values['desde'] = ($pag - 1) * $this->_values['rpp'];
		$this->_values['numpags'] = $numpags;
		$this->_params['pag'] = $pag;
	}
	/**
	 * Sólo se usa desde el tpl,
	 * para crear las cabeceras de las columnas
	 *
	 */
	public function CompletarCampos()
	{
		if(is_array($this->_values['regs'][0]))
		{
			$fields = array_keys($this->_values['regs'][0]);
			$fields_size = sizeof($fields);
			if(!is_array($this->_params['campos']))
			{
				$this->_params['campos'] = array();
			}
			for($c=0; $c<$fields_size; $c++)
			{
				if(!isset($this->_params['campos'][$fields[$c]]))
				{
					$this->_params['campos'][$fields[$c]] = array("label"=>$fields[$c], "orderby"=>$fields[$c], "hidden"=>$this->_params['camposEstrictos']);
				}
				elseif(!isset($this->_params['campos'][$fields[$c]]['orderby']) || $this->_params['campos'][$fields[$c]]['orderby'] === true)
				{
					$this->_params['campos'][$fields[$c]]['orderby'] = $fields[$c];
				}
			}
		}
	}
	/**
	 * Sólo se usa el tpl,
	 * para coger el conjunto de parámetros enviados por el formulario
	 *
	 * @return array
	 */
	public function GetRequested()
	{
		$request = $this->_GetRequest();
		$keys = array_keys($request);
		$requested = array();
		for($c=0,$size=sizeof($request); $c<$size; $c++)
		{
			$name = $keys[$c];
			//Desechamos rehacer operaciones (reg, upd, del,...)
			if($name != "op")
			{
				$requested[] = array("name"=>$name, "value"=>$request[$name]);
			}
		}
		if(!in_array("filtro_campo", $keys))
		{
			$requested[] = array("name"=>"filtro_campo", "value"=>$this->_params['filtro_campo']);
		}
		if(!in_array("filtro_valor", $keys))
		{
			$requested[] = array("name"=>"filtro_valor", "value"=>$this->_params['filtro_valor']);
		}
		if(!in_array("filtro_exacto", $keys))
		{
			$requested[] = array("name"=>"filtro_exacto", "value"=>$this->_params['filtro_exacto']);
		}
		if(!in_array("pag", $keys))
		{
			$requested[] = array("name"=>"pag", "value"=>$this->_params['pag']);
		}
		if(!in_array("por", $keys))
		{
			$requested[] = array("name"=>"por", "value"=>"");
			$requested[] = array("name"=>"modo", "value"=>"");
		}

		return $requested;
	}
	/**
	 * Sólo se usa desde el tpl,
	 * para obtener el html del control orden ascendente
	 *
	 * @param string $campo_orderby
	 * @return string
	 */
	public function GetOrderASC($campo_orderby)
	{
		$orderBy = explode(" ", $this->_values['orderBy']);
		$activo = ($orderBy[0] == $campo_orderby && $orderBy[1] == "ASC");
		return $this->_params['usarImg'] ?
			"<img src=\"{$this->_params['img_dir']}".($activo ? $this->_params['img_orderASC_on'] : $this->_params['img_orderASC_off'])."\" alt=\"".l('Orden ascendente')."\" title=\"".l('Orden ascendente')."\" />"
			:
			l('Orden ascendente');
	}
	/**
	 * Sólo se usa desde el tpl,
	 * para obtener el html del control orden descendente
	 *
	 * @param string $campo_orderby
	 * @return string
	 */
	public function GetOrderDESC($campo_orderby)
	{
		$orderBy = explode(" ", $this->_values['orderBy']);
		$activo = ($orderBy[0] == $campo_orderby && $orderBy[1] == "DESC");
		return $this->_params['usarImg'] ?
			"<img src=\"{$this->_params['img_dir']}".($activo ? $this->_params['img_orderDESC_on'] : $this->_params['img_orderDESC_off'])."\" alt=\"".l('Orden descendente')."\" title=\"".l('Orden descendente')."\" />"
			:
			l('Orden descendente');
	}
	/**
	 * Sólo se usa desde el tpl,
	 * para obtener el html del control página anterior
	 *
	 * @return string
	 */
	public function GetAnterior()
	{
		$label = l('Anterior');

		return $this->_params['usarImg'] ? "<img src=\"{$this->_params['img_dir']}{$this->_params['img_anterior']}\" alt=\"{$label}\" title=\"{$label}\" />" : $label;
	}
	/**
	 * Sólo se usa desde el tpl,
	 * para obtener el html del control página siguiente
	 *
	 * @return string
	 */
	public function GetSiguiente()
	{
		$label = l('Siguiente');

		return $this->_params['usarImg'] ? "<img src=\"{$this->_params['img_dir']}{$this->_params['img_siguiente']}\" alt=\"{$label}\" title=\"{$label}\" />" : $label;
	}
	/**
	 * Sólo se usa desde el tpl,
	 * para obtener el html del control página primera
	 *
	 * @return string
	 */
	public function GetPrimera()
	{
		$label = l('Primera');

		return $this->_params['usarImg'] ? "<img src=\"{$this->_params['img_dir']}{$this->_params['img_primera']}\" alt=\"{$label}\" title=\"{$label}\" />" : $label;
	}
	/**
	 * Sólo se usa desde el tpl,
	 * para obtener el html del control página última
	 *
	 * @return string
	 */
	public function GetUltima()
	{
		$label = l('Última');

		return $this->_params['usarImg'] ? "<img src=\"{$this->_params['img_dir']}{$this->_params['img_ultima']}\" alt=\"{$label}\" title=\"{$label}\" />" : $label;
	}
	/**
	 * Sólo se usa desde el tpl,
	 * para obtener la etiqueta correspondiente a resultados
	 *
	 * @return string
	 */
	public function GetEtiqueta()
	{
		if($this->_values['total'] == 1)
		{
			return $this->_params['etiqueta'];
		}
		else
		{
			return Pluralizador::Plural($this->_params['etiqueta'], true, isset($_SESSION['SWPHP_IDIOMA_ACTUAL_CODIGOISO']) ? $_SESSION['SWPHP_IDIOMA_ACTUAL_CODIGOISO'] : SWPHP_IDIOMA_POR_DEFECTO_CODIGOISO);
		}
	}
}
