<?php
/**
 * Clase con funciones estáticas genéricas del sistema SWPHP para usar con Xajax
 *
 * Jose Carlos Cruz Parra
 * programadorphpfreelance.com
 * 2011-10-30
 *
 * SIMPLE WEB PHP EDICION ADMINISTRADA
 *
 * inc/classes/SWPHP_Xajax.class.php
 *
 */
abstract class SWPHP_Xajax
{
	/**
	 * Cambia el valor de un campo de tipo 0/1 y en consecuencia cambia el title e icono del botoncito
     * $id puede ser un array:
     * 0 - Id. del elemento de la tabla en la bd
     * 1 - Id. del elemento en el listado html
	 *
	 * @param xajaxResponse $response
	 * @param mixed $id
	 * @param string $tabla
	 * @param string $campo
	 * @param string $aLabel
	 * @param string $titleOn
	 * @param string $titleOff
	 * @param string $iconOn
	 * @param string $iconOff
	 * @param array $otrosCampos
	 * @param int $iconWidth
	 * @return xajaxResponse
	 */
	static function _Switch($response, $id, $tabla, $campo='activo', $aLabel='aActivar', $titleOn='Desactivar', $titleOff='Activar', $iconOn='green_button.png', $iconOff='red_button.png', $otrosCampos=array(), $iconWidth=16)
	{
		global $db;

		if(is_null($response))
		{
			$response = new xajaxResponse();
		}

		$campoId = 'id';

        if(is_array($id))
        {
			include_once SWPHP_COMMON_DIR.'functions/arrays.php';

			if(array_is_num($id))
			{
				$idHtml = $id[1];
				$idVal = intval($id[0]);
			}
			elseif(sizeof($id) == 1)
			{
				foreach($id as $key=>$val)
				{
					$campoId = $key;
					$idHtml = intval($val);
					$idVal = $idHtml;

					break;
				}
			}
			else
			{
				PIGUI::Msg('Valor no esperado para $id', 'error');
			}
        }
        else
        {
            $idHtml = intval($id);
            $idVal = $idHtml;
        }

		if(SWPHP::_Switch($tabla, $id, $campo, $otrosCampos))
		{
			$activo = $db->GetValueQuery("SELECT {$campo} FROM {$tabla} WHERE `{$campoId}` = '{$idVal}'", false);

			$response->script("gebi('{$aLabel}{$idHtml}').title = '".($activo ? $titleOn : $titleOff)."'");
			$response->script("gebi('{$aLabel}{$idHtml}').innerHTML = '".SWPHP::_Icon($activo ? $iconOn : $iconOff, $iconWidth)."'");
		}

		return $response;
	}
	/**
	 * Elimina un registro
	 *
	 * @param xajaxResponse $response
	 * @param int $id
	 * @param string $tabla
	 * @param string $que
	 * @param string $sqlOnSuccess
	 * @param string $phpOnSuccess
	 * @param string $aLabel
	 * @return xajaxResponse
	 */
	static function _Eliminar($response, $id, $tabla, $que='registro', $sqlOnSuccess='', $phpOnSuccess='', $aLabel='aEliminar')
	{
		if(is_null($response))
		{
			$response = new xajaxResponse();
		}

		if(!is_array($id))
		{
			$id = intval($id);
		}

		if(SWPHP::_OpDel($tabla, $id, $que, false, $sqlOnSuccess, $phpOnSuccess))
		{
			if(is_array($id))
			{
				include_once SWPHP_COMMON_DIR.'functions/arrays.php';

				$aux = array_ToString($id, '_');
			}
			else
			{
				$aux = $id;
			}

			$response->script("gebi('{$aLabel}{$aux}').parentNode.parentNode.style.display = 'none'; var key = gebi('{$aLabel}{$aux}').parentNode.parentNode.parentNode.parentNode.id.replace('paginador_', ''); \$j('#pag_total_'+key).text(parseInt(\$j('#pag_total_'+key).text())-1);");
			$response = SWPHP_Xajax::_VolcarMensajes($response, true);
		}
		else
		{
			$response = SWPHP_Xajax::_VolcarMensajes($response, false);
		}

		return $response;
	}
	/**
	 * Ejecuta una función
	 *
	 * @param xajaxResponse $response
	 * @param string $function
	 * @param mixed $params
	 * @param string $jsOnSuccess
	 * @return xajaxResponse
	 */
	static function _Func($response, $function, $params=array(), $jsOnSuccess='')
	{
		if(is_null($response))
		{
			$response = new xajaxResponse();
		}

		$ok = false;

		if(function_exists($function))
		{
			$str = $function;
			if(is_array($params))
			{
				$sep = '(';
				foreach($params as $key=>$param)
				{
					$str .= $sep."\$params[\$key]";
					$sep = ',';
				}
				$str .= ')';
			}
			else
			{
				$str .= "(\$params)";
			}

			if(eval("return {$str};"))
			{
				if($jsOnSuccess)
				{
					$response->script($jsOnSuccess);
				}

				$ok = true;
			}
		}
		else
		{
			SWPHP::_Msg('No se encuentra la función '.$function, 'error');
		}

		$response = SWPHP_Xajax::_VolcarMensajes($response, $ok);

		return $response;
	}
	/**
	 * Muestra los mensajes acumulados en $swphp_mensajes
	 *
	 * @param xajaxResponse $response
	 * @param bool $soloSiHayMasDeUno
	 * @return xajaxResponse
	 */
	static function _VolcarMensajes($response=null, $soloSiHayMasDeUno=false)
	{
		global $swphp_mensajes;

		if(is_null($response))
		{
			$response = new xajaxResponse();
		}

		if(!$soloSiHayMasDeUno ||sizeof($swphp_mensajes) > 1)
		{
			foreach($swphp_mensajes as $mensaje)
			{
				$response->alert(strtoupper($mensaje['clase'])."\n\n".addcslashes(strip_tags($mensaje['texto']), "'")."\n\n");
			}
		}

		return $response;
	}
	/**
	 * Recarga un select con elementos tomados de una tabla
	 *
	 * @param xajaxResponse $response
	 * @param string $select_id
	 * @param string $query
	 * @param array $option
	 * @param mixed $seleccionado
	 * @param array $optionNoHay
	 * @return xajaxResponse
	 */
	static function _CargarSelectQuery($response, $select_id, $query, $option=null, $seleccionado=null, $optionNoHay=array('','[no hay]'), $function=false, $seleccionarSiEsUno=true)
	{
		global $db;

		if(is_null($response))
		{
			$response = new xajaxResponse();
		}

		$js = "select_Empty('{$select_id}');";

		$rows = $db->GetArrayForHTMLSelectQuery($query, null, false);
		$size = sizeof($rows);
        if(is_array($rows) && $size > 0)
		{
            if(($size > 1 || !$seleccionarSiEsUno || (!empty($seleccionado) && !key_exists($seleccionado, $rows))) && $option)
            {
                $js .= sprintf("select_AddOption('%s','%s','%s');", $select_id, addcslashes($option[1], "'"), $option[0]);
            }

			foreach($rows as $key=>$value)
			{
                $js .= sprintf("select_AddOption('%s','%s','%s',%u);", $select_id, addcslashes($function && function_exists($function) ? $function($value) : $value, "'"), $key, $key == $seleccionado ? '1' : '0');
			}

            //$js .= "select_Order('{$select_id}');";
		}
		elseif($optionNoHay)
		{
            $js .= sprintf("select_AddOption('%s','%s','%s');", $select_id, addcslashes($optionNoHay[1], "'"), $optionNoHay[0]);
		}

		$response->script($js);

		return $response;
	}
	/**
	 * Recarga un select con elementos tomados de una tabla
	 *
	 * @param xajaxResponse $response
	 * @param string $select_id
	 * @param string $index_field
	 * @param string $text_field
	 * @param string $table
	 * @param string $filter
	 * @param array $option
	 * @param mixed $seleccionado
	 * @param array $optionNoHay
	 * @return xajaxResponse
	 */
	static function _CargarSelect($response, $select_id, $index_field, $text_field, $table, $filter='', $option=null, $seleccionado=null, $optionNoHay=array('','[no hay]'), $function=false, $addcomilla=true)
	{
		$query = $addcomilla ? "SELECT `$index_field`, `$text_field` FROM `$table` $filter" : "SELECT $index_field, $text_field FROM $table $filter";
		return SWPHP_Xajax::_CargarSelectQuery($response, $select_id, $query, $option, $seleccionado, $optionNoHay, $function);
	}
	/**
	 * Elimina un archivo y oculta la capa
	 *
	 * @param xajaxResponse $response
	 * @param string $file
	 * @param string $dir
	 * @param string $idcapa
	 * @param array $subdirs
	 * @return xajaxResponse
	 */
	static function _EliminarArchivo($response, $file, $dir, $idcapa=false, $subdirs=array())
	{
		$ok = false;

		if(is_null($response))
		{
			$response = new xajaxResponse();
		}

		ponerBarra($dir);

		if(is_file($dir.$file))
		{
			if(unlink($dir.$file))
			{
				foreach($subdirs as $subdir)
				{
					$subdir = ponerBarra($subdir);
					unlink($dir.$subdir.$file);
				}

				$ok = true;
			}
		}
		else
		{
			$ok = true;
		}

		if($ok)
		{
			if($idcapa)
			{
				$response->script("\$j('#{$idcapa}').hide('slow')");
			}
			else
			{
				SWPHP::_Msg(addcslashes($file, "'")."\n\nArchivo eliminado correctamente", "check");
			}
		}
		else
		{
			SWPHP::_Msg(addcslashes($file, "'")."\n\nNo se ha podido eliminar el archivo", "error");
		}

		return SWPHP_Xajax::_VolcarMensajes($response);
	}
}
