<?php
/**
 * Jose Carlos Cruz Parra
 * programadorphpfreelance.com
 * 2011-03-14
 *
 * SIMPLE WEB PHP EDICION ADMINISTRADA
 *
 * inc/modulos/noticias/admin/functions.inc.php
 *
 */

/**
 * Valida los datos de un registro de noticia
 *
 * @param array $data
 * @return bool
 */
function noticias_validarDatos($data)
{
	global $swphp_idiomas;

	$ok = true;

	if(!isset($data['fecha']) || $data['fecha'] == '')
	{
		SWPHP::_Msg("Falta el dato: Fecha", "error");
		$ok = false;
	}

	$requeridos = array(
		'titulo'=>'Título',
		'introduccion'=>'Introducción',
		'contenido'=>'Contenido',
		);

	foreach($requeridos as $key=>$nombre)
	{
		if(!isset($data['idiomas'][SWPHP_IDIOMA_POR_DEFECTO][$key]) || $data['idiomas'][SWPHP_IDIOMA_POR_DEFECTO][$key] == '')
		{
			SWPHP::_Msg("Falta el dato: {$nombre} ({$swphp_idiomas[SWPHP_IDIOMA_POR_DEFECTO]['nombre']})", "error");
			$ok = false;
		}
	}

	return $ok;
}
/**
 * Prepara los datos para insertar o actualizar una noticia en la base de datos
 *
 * @param array $data
 * @param int $id
 * @return array
 */
function noticias_prepararDatos($data, $id=0)
{
	$data['mostrarimagen'] = isset($data['mostrarimagen']) && $data['mostrarimagen'];
	$data['imagenampliable'] = isset($data['imagenampliable']) && $data['imagenampliable'];

	foreach($data as $key=>$value)
	{
		switch($key)
		{
			case "mostrarimagen":
			case "imagenampliable":
				$data[$key] = $value ? 1 : 0;
				break;
			case "imagen":
				//nada
				break;
			case "fecha":
				include_once(SWPHP_COMMON_DIR.'functions/date.php');
				$data[$key] = fechadmY2Ymd($value);
				break;
			case "fuente_id":
				$data[$key] = intval($value);
				break;
			default:
				//Cualquier otro dato lo descarto
				unset($data[$key]);
				break;
		}
	}

	$ahora = date('Y-m-d H:i:s');
	if($id == 0)
	{
		$data['fechayhoraregistro'] = $ahora;
		$data['user_idregistro'] = $_SESSION['user']['id'];
	}
	$data['fechayhoraultima'] = $ahora;
	$data['user_idultima'] = $_SESSION['user']['id'];

	return $data;
}
/**
 * Elimina la imagen asociada a una noticia
 *
 * @param int $id
 */
function noticias_eliminarImagen($id)
{
	global $db;

	$imagen = $db->GetValueQuery("SELECT imagen FROM ".SWPHP_DBPREFIX."noticias WHERE id = '{$id}'", false);

	$imagedir = dirname($imagen);
	$imagefile = basename($imagen);

	$arr[] = $imagen;
	$arr[] = "{$imagedir}/thumbs/{$imagefile}";
	$arr[] = "{$imagedir}/preview/{$imagefile}";

	foreach($arr as $img)
	{
		if($imagen && is_file($img) && !unlink($img))
		{
			SWPHP::_Msg("No se pudo eliminar <b>{$img}</b>", "aviso");
		}
	}

	$db->Execute("UPDATE ".SWPHP_DBPREFIX."noticias SET imagen = '' WHERE id = '{$id}'");
}
/**
 * Elimina una noticia
 *
 * @param int $id
 */
function noticias_eliminarNoticia($id)
{
	global $db;

	if($db->Execute("DELETE FROM ".SWPHP_DBPREFIX."noticias WHERE id = '{$id}'"))
	{
		noticias_eliminarImagen($id);

		$db->Execute("DELETE FROM ".SWPHP_DBPREFIX."noticias_idiomas WHERE noticia_id = '{$id}'");

		return true;
	}

	return false;
}
/**
 * Prepara los datos para insertar o actualizar datos de una noticia en idioma
 *
 * @param array $idiomas
 * @param int $idioma_id
 * @param int $id
 * @return array
 */
function noticias_prepararDatosIdioma($idiomas, $idioma_id, $id=0)
{
	global $db;

	$friendly = empty($idiomas[$idioma_id]['friendly']) && !empty($idiomas[SWPHP_IDIOMA_POR_DEFECTO]['friendly']) ? '' : $idiomas[$idioma_id]['friendly'];

	$data = SWPHP::_PrepararDatosIdioma($idiomas, $idioma_id, array('titulo','introduccion','contenido','friendly','keywords','tags','urlimagen'));

	$data['friendly'] = string2friendly($friendly ? $friendly : $data['titulo']);

	$c = 1;
	//Evitamos urls repetidas
	$friendly = $data['friendly'];
	while($db->ExistsQuery("SELECT ni.friendly FROM ".SWPHP_DBPREFIX."noticias AS n JOIN ".SWPHP_DBPREFIX."noticias_idiomas AS ni ON n.id = ni.noticia_id WHERE n.id != '{$id}' AND ni.idioma_id = '{$idioma_id}' AND ni.friendly = '{$data['friendly']}'"))
	{
		$data['friendly'] = "{$friendly}-{$c}";
		$c++;
	}

	if($id)
	{
		$data['noticia_id'] = $id;
	}
	$data['idioma_id'] = $idioma_id;

	return $data;
}
/**
 * Obtiene un array asociativo con las fuentes de noticias,
 * donde los índices son los id y los valores los nombres
 *
 * @return array
 */
function noticias_getFuentes()
{
	global $db;

	return $db->GetArrayForHTMLSelect('id', 'nombre', SWPHP_DBPREFIX.'fuentesnoticias', 'ORDER BY nombre', array(0, '[Sin fuente]'));
}
